/************************************************************
* $Id: nfiles.c,v 1.8 2009/08/11 15:47:44 mechanoid Exp mechanoid $
*
*  ART1 neural net based classifier of BMP images
*
*       OS : FreeBSD 7.2
* COMPILER : GNU project C and C++ Compiler  v. 4.2.1
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
*
*    http://www.mechanoid.kiev.ua
*  e-mail : nn@mechanoid.kiev.ua
* 
************************************************************/
#include <gdk/gdk.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>

#include "nnet.h"
#include "nfiles.h"


/* - - - - - - - - - - - - - - - - - - -
* load nnet input from BMP file         
*/
int input_load(char filename[]) {
   GdkPixbuf *pixbuf = NULL;
   GError  *err = NULL;
   guchar *buffer;
   int i,j,k;
   
   pixbuf= gdk_pixbuf_new_from_file(filename,&err);
   if(err!=NULL) { fprintf (stderr, err->message); g_error_free(err); return 3; }
   
   g_return_val_if_fail(pixbuf != NULL, 1);
   buffer=gdk_pixbuf_get_pixels(pixbuf);
   g_return_val_if_fail(buffer != NULL, 2);
   
   for( k=0,j=0 ; j<IMG_HEIGHT ; j++ ){
	  for( i=0 ; i<IMG_ROWS ; i++ ) { 
		 layer_0[k++]=buffer[i+(IMG_ROWS+IMG_NROWS)*j]>0?LO:HI;
	  }
   }
   gdk_pixbuf_unref(pixbuf);
   
   return 0; 
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
* save level 2 neuron state and weight matrices to file                                    
*/
int nnet_memory_save(char filename[]) {
   data_t reserved[SIZE_LAYER_2];
   ssize_t sz;
   int i, file;

   for( i=0 ; i<SIZE_LAYER_2 ; i++ ) { reserved[i]=layer_2[i].reserved; }

   file=open(filename,O_WRONLY|O_TRUNC|O_CREAT);
   
   if(file<0){ fprintf(stderr,"%s : %s\n",filename,strerror(errno)); return 1; }

   sz=write(file, reserved, SIZE_LAYER_2*sizeof(data_t));
   if( sz!= SIZE_LAYER_2*sizeof(data_t) ) {
  	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
	  close(file);
	  return 2;
   }
   sz=write(file, weight_layer_1, SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t));
   if(sz!= SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t)) {
  	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
	  close(file);
	  return 2;
   }
   sz=write(file, weight_layer_2, SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t));
   if(sz!= SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t)) {
  	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
	  close(file);
	  return 3;
   }
   
   close(file);

   if(chmod(filename,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) {
	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
   }
   return 0;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* load level 2 neuron state and weight matrices from file                                    
*/
int nnet_memory_load(char filename[]) {
   data_t reserved[SIZE_LAYER_2];
   ssize_t sz;
   int i, file;

   file=open(filename,O_RDONLY);
   if(file<0) {
	  if(errno==ENOENT) {
		 return 0;
	  }
	  fprintf(stderr,"%s : %s\n",filename,strerror(errno)); 
	  return 1; 
   } 

   sz=read(file, reserved, SIZE_LAYER_2*sizeof(data_t));
   if(sz!= SIZE_LAYER_2*sizeof(data_t)) {
  	  fprintf(stderr,"%s : %s\n", filename, strerror(errno));
	  close(file);
	  return 2;
   }
   sz=read(file, weight_layer_1, SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t));
   if(sz!= SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t)) {
  	  fprintf(stderr,"%s : %s\n", filename, strerror(errno));
	  close(file);
	  return 2;
   }
   sz=read(file, weight_layer_2, SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t));
   if(sz!= SIZE_LAYER_1*SIZE_LAYER_2*sizeof(weight_t)) {
  	  fprintf(stderr,"%s : %s\n", filename, strerror(errno));
	  close(file);
	  return 2;
   }

   close(file);

   for( i=0 ; i<SIZE_LAYER_2 ; i++ ) { layer_2[i].reserved=reserved[i]; }

   return 0;
}

